@@ -1,4 +1,7 @@
 """The Python package of the Oxynet project"""
 
 from . import utilities
-from .utilities import *
\ No newline at end of file
+from .utilities import *
+
+from . import testing
+from .testing import *
\ No newline at end of file
